@@ -22,6 +22,7 @@
  */
 
 #include "FrameHelper.h"
+#include "AwtKeyToDbusmenuString.h"
 
 #include <com/sun/star/awt/KeyEvent.hpp>
 #include <com/sun/star/awt/SystemDependentXWindow.hpp>
@@ -33,6 +34,7 @@
 #include <com/sun/star/container/XNameAccess.hpp>
 #include <com/sun/star/awt/XPopupMenu.hpp>
 #include <com/sun/star/awt/XMenuExtended.hpp>
+#include <com/sun/star/awt/XMenuListener.hpp>
 #include <com/sun/star/awt/XPopupMenuExtended.hpp>
 #include <com/sun/star/frame/XController.hpp>
 #include <com/sun/star/frame/XComponentLoader.hpp>
@@ -70,6 +72,8 @@ using namespace ::com::sun::star;
 using com::sun::star::awt::KeyEvent;
 using com::sun::star::awt::SystemDependentXWindow;
 using com::sun::star::awt::XMenuExtended;
+using com::sun::star::awt::XMenuListener;
+using com::sun::star::awt::MenuEvent;
 using com::sun::star::awt::XPopupMenu;
 using com::sun::star::awt::XPopupMenuExtended;
 using com::sun::star::awt::XSystemDependentWindowPeer;
@@ -480,9 +484,48 @@ FrameHelper::dispatchCommand (OUString command)
 	OUString target = OUString::createFromAscii ("");
 	Reference < XDispatchHelper > xdh (xMSF->createInstance(OUString(RTL_CONSTASCII_USTRINGPARAM("com.sun.star.frame.DispatchHelper"))),
 	                                   UNO_QUERY);
-	
+	// This is a special case for the recentfilelist
 	if (command.matchAsciiL ("vnd.sun.star.popup:RecentFileList", 33, 0))
-	    target = OUString::createFromAscii ("_blank");		
+	{
+	    target = OUString::createFromAscii ("_default");
+	    
+		Reference < XPropertySet > xMSFProps (xMSF, UNO_QUERY);	    	    
+		Reference <XComponentContext> xContext (xMSFProps->getPropertyValue (OUString::createFromAscii ("DefaultContext")),
+		                                        UNO_QUERY);
+		Reference < XPopupMenuController > xRFC (xPCF->createInstanceWithArgumentsAndContext(OUString::createFromAscii (".uno:RecentFileList"),
+                                                                                             args,
+		                                                                                     xContext),
+		                                         UNO_QUERY);
+		Reference < XMenuListener > xML (xRFC, UNO_QUERY);
+		
+		Reference < XPopupMenu > xPO (xMSF->createInstance(OUString::createFromAscii ("stardiv.Toolkit.VCLXPopupMenu")),
+		                              UNO_QUERY);
+		
+		if (xRFC.is () && xPO.is ())
+		{
+			xRFC->setPopupMenu (xPO);
+			xRFC->updatePopupMenu ();
+			Reference < XMenu > subMenu (xPO, UNO_QUERY);
+			Reference < XMenuExtended > subMenuEx (xPO, UNO_QUERY);
+			
+			//We need to find the item idd
+			for (int i = 0; i < subMenu->getItemCount (); i++)
+			{
+				int id = subMenu->getItemId (i);
+				
+				if (subMenuEx->getCommand (id).equals (command))
+				{
+					MenuEvent mev;
+					mev.MenuId = id;
+					
+					xML->select (mev);
+				}
+			}
+		}
+		
+		return;
+	}
+
 	
 	if (command.matchAsciiL ("private:factory/", 16, 0))
 	    target = OUString::createFromAscii ("_blank");
@@ -620,26 +663,26 @@ FrameHelper::rebuildMenu (Reference < XMenu >  xMenu,
 
 		if (kev.KeyCode != 0)  //KeyCode must have a value
 		{
-			guint keycode;
-			GdkModifierType mod = (GdkModifierType)0;
-		
-			if (awt::KeyModifier::SHIFT & kev.Modifiers)
-				mod = (GdkModifierType)(mod | GDK_SHIFT_MASK);
-			if (awt::KeyModifier::MOD2 & kev.Modifiers)
-				mod = (GdkModifierType)(mod | GDK_MOD1_MASK); //Alt key
-			if (awt::KeyModifier::MOD1 & kev.Modifiers || awt::KeyModifier::MOD3 & kev.Modifiers)
-				mod = (GdkModifierType)(mod | GDK_CONTROL_MASK);
+			GVariantBuilder builder;
+			const gchar* keystring = AwtKeyToDbusmenuString(kev.KeyCode);
 
-			if (kev.KeyCode >= awt::Key::F1 && kev.KeyCode <= awt::Key::F1)
-			{
-				keycode = (guint)(kev.KeyCode - awt::Key::F1) + (guint)0xffbe;
-				dbusmenu_menuitem_property_set_shortcut (item, keycode, mod);
-			}
-			else if (kev.KeyCode >= awt::Key::A && kev.KeyCode <= awt::Key::Z)
-			{
-				keycode = (guint)(kev.KeyCode - awt::Key::A) + (guint)'a';
-				dbusmenu_menuitem_property_set_shortcut (item, keycode, mod);
-			}
+			g_variant_builder_init(&builder, G_VARIANT_TYPE_ARRAY);
+
+			if (awt::KeyModifier::SHIFT & kev.Modifiers)
+				g_variant_builder_add(&builder, "s", DBUSMENU_MENUITEM_SHORTCUT_SHIFT);
+			if (awt::KeyModifier::MOD2 & kev.Modifiers)
+				g_variant_builder_add(&builder, "s", DBUSMENU_MENUITEM_SHORTCUT_ALT);
+			if (awt::KeyModifier::MOD1 & kev.Modifiers || awt::KeyModifier::MOD3 & kev.Modifiers)
+				g_variant_builder_add(&builder, "s", DBUSMENU_MENUITEM_SHORTCUT_CONTROL);
+
+			g_variant_builder_add(&builder, "s", keystring);
+
+			GVariant * inside = g_variant_builder_end(&builder);
+			g_variant_builder_init(&builder, G_VARIANT_TYPE_ARRAY);
+			g_variant_builder_add_value(&builder, inside);
+
+			GVariant * outsidevariant = g_variant_builder_end(&builder);
+			dbusmenu_menuitem_property_set_variant(item, DBUSMENU_MENUITEM_PROP_SHORTCUT, outsidevariant);
 		}
 
 		MenuItemInfo* commInfo = (MenuItemInfo*)g_hash_table_lookup (commandsInfo, (gconstpointer)command.getStr());
@@ -842,27 +885,19 @@ FrameHelper::findShortcutForCommand (OUString command)
 
 {
 	KeyEvent kev;
-	OString com = OUStringToOString (command, RTL_TEXTENCODING_ASCII_US);
 
 	Sequence < OUString > commands (1);
 	commands[0] = command;
-
-	//TODO: The Help menu needs a special case for F1
-	//TODO: The Repeat command should always be taken from the global configuration
 	
 	try 
 	{
 		Sequence < Any > evs = docAccelConf->getPreferredKeyEventsForCommandList (commands);
-		
 
 		for (int j = 0; j < evs.getLength (); j++)
 		{
 			KeyEvent ev;
 			if (evs[j] >>= ev)
-			{
-				//g_debug ("Document:\t%x - %x - %x - %x", ev.KeyCode, ev.KeyChar, ev.KeyFunc, ev.Modifiers);
 				return ev;
-			}
 		}
 	}
 	catch (...)
@@ -876,10 +911,7 @@ FrameHelper::findShortcutForCommand (OUString command)
 		{
 			KeyEvent ev;
 			if (evs[j] >>= ev)
-			{
-				//g_debug ("Module:\t%x - %x - %x - %x", ev.KeyCode, ev.KeyChar, ev.KeyFunc, ev.Modifiers);
 				return ev;
-			}
 		}
 	}
 	catch (...)
@@ -892,13 +924,17 @@ FrameHelper::findShortcutForCommand (OUString command)
 		{
 			KeyEvent ev;
 			if (evs[j] >>= ev)
-			{
-				//g_debug ("Global:\t%x - %x - %x - %x", ev.KeyCode, ev.KeyChar, ev.KeyFunc, ev.Modifiers);
 				return ev;
-			}
 		}
 	}
 	catch (...)
 	{}
+
+	//NOTE: For some reason this item does not return its shortcut. Setting manually:
+	if (command.equals (OUString::createFromAscii (".uno:HelpIndex")))
+	{
+		kev.KeyCode = awt::Key::F1;
+	}
+	
 	return kev;
 }
